# -*- coding: utf-8 -*-

from radish.colorful import colorful


class RadishError(Exception):
    pass


class FeatureFileNotValidError(RadishError):
    """Raised when a feature file could not be parsed"""
    def __init__(self, feature_file):
        self._feature_file = feature_file

    def __str__(self):
        return colorful.red("The feature file '%s' is not valid" % (self._feature_file))


class StepLoadingError(RadishError):
    """Raised when a step decorator regex is not valid to compile"""
    def __init__(self, regex):
        self._regex = regex

    def __str__(self):
        return colorful.red("The step decorator regex '%s' is not valid to compile" % (self._regex))


class StepDefinitionNotFoundError(RadishError):
    """Raised when a step definition could not be found"""
    def __init__(self, step_sentence, step_filename):
        self._step_sentence = step_sentence
        self._step_filename = step_filename

    def __str__(self):
        return colorful.red("The step definition for '%s' from file '%s' could not be found" % (self._step_sentence, self._step_filename))


class FeatureFileNotFoundError(RadishError):
    """Raised when a feature file could not be found"""
    def __init__(self, path):
        self._path = path

    def __str__(self):
        return colorful.red("The feature file '%s' could not be found" % (self._path))


class BasedirNotFoundError(RadishError):
    """Raised when the basedir could not be found"""
    def __init__(self, basedir):
        self._basedir = basedir

    def __str__(self):
        return colorful.red("The basedir '%s' could not be found" % (self._basedir))


class StepDefinitionFileNotFoundError(RadishError):
    """Raised when the step definition file could not be found"""
    def __init__(self, root, pattern):
        self._root = root
        self._pattern = pattern

    def __str__(self):
        return colorful.red("The step definition file '%s' could not be found in the directory '%s'" % (self._pattern, self._root))


class NoMetricUtilFoundError(RadishError):
    """Raised if metric should be shown but no utils hook was found"""
    def __str__(self):
        return colorful.red("Could not found util hook to show metric - implement: utils( \"show_metrics\" )")


class ValidationError(RadishError):
    """Raised when a validation error occured during run"""
    def __init__(self, msg):
        self._msg = msg

    def __str__(self):
        return colorful.bold_red(self._msg)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import radish

import os
import sys
import time
import optparse


def main():
    parser = optparse.OptionParser(
        description="radish is a smart 'Behavior Driven Developement'-Tool written in python\nVersion: %s" % radish.version.__version__,
        version=radish.version.__version__,
        epilog="(C) Copyright 2013 by Timo Furrer <tuxtimo@gmail.com>"
    )
    parser.add_option(
        "-b", "--basedir",
        dest="basedir",
        default=os.path.join(os.getcwd(), "radish"),
        help="The basedir is used to locate the steps.py and terrain.py"
    )
    parser.add_option(
        "-a", "--abort-fail",
        dest="abort_fail",
        action="store_true",
        help="If one feature file fails this option will stop the execution"
    )
    parser.add_option(
        "-m", "--marker",
        dest="marker",
        default=int(time.time()),
        help="A specific marker for the step loggings"
    )
    parser.add_option(
        "-d", "--dry-run",
        dest="dry_run",
        action="store_true",
        help="Executes a dry run to validate steps"
    )
    parser.add_option(
        "-x", "--xunit-file",
        dest="xunit_file",
        default=None,
        help="Location where to write to JUnit xml report file"
    )
    parser.add_option(
        "--split-xunit",
        dest="split_xunit",
        action="store_true",
        help="If you have specified the -x option this option will split the xunit xml file and create one file per feature file; the -x argument specifies the location"
    )
    parser.add_option(
        "-p", "--profile",
        dest="profile",
        default=None,
        help="Define profile"
    )
    parser.add_option(
        "-t", "--with-traceback",
        dest="with_traceback",
        action="store_true",
        help="Print traceback if a step fails"
    )
    parser.add_option(
        "--no-colors",
        dest="no_colors",
        action="store_true",
        help="Do not print in colors"
    )
    parser.add_option(
        "--no-line-jump",
        dest="no_line_jump",
        action="store_true",
        help="Do not jump up lines to rewrite in another color"
    )
    parser.add_option(
        "--no-overwrite",
        dest="no_overwrite",
        action="store_true",
        help="Do not overwrite step. Step will always be in the color indicating that it's still executing, so recommended is combining with --no-colors"
    )
    parser.add_option(
        "--no-duration",
        dest="no_duration",
        action="store_true",
        help="Do not print duration after execution"
    )
    parser.add_option(
        "--no-numbers",
        dest="no_numbers",
        action="store_true",
        help="Do not print numbers before feature, scenario and step sentences"
    )
    parser.add_option(
        "--no-indentation",
        dest="no_indentation",
        action="store_true",
        help="Do not print any indentation before sentences"
    )
    parser.add_option(
        "--no-skipped-steps",
        dest="no_skipped_steps",
        action="store_true",
        help="Do not print skipped steps"
    )
    parser.add_option(
        "--with-section-names",
        dest="with_section_names",
        action="store_true",
        help="print section name before feature and scenario sentences"
    )
    parser.add_option(
        "--show-metrics",
        dest="show_metrics",
        action="store_true",
        help="Show metrics of given feature files"
    )

    options, args = parser.parse_args()

    exitCode = 0
    try:
        # initialize config object
        cf = radish.Config()
        cf.no_colors = options.no_colors
        cf.no_line_jump = options.no_line_jump
        cf.SetBasedir(radish.FileSystemHelper.expand(options.basedir))
        cf.feature_files = args
        cf.abort_fail = options.abort_fail
        cf.marker = options.marker
        cf.dry_run = options.dry_run
        cf.xunit_file = options.xunit_file
        cf.split_xunit = options.split_xunit
        cf.profile = options.profile
        cf.no_numbers = options.no_numbers
        cf.no_indentation = options.no_indentation
        cf.no_overwrite = options.no_overwrite
        cf.no_duration = options.no_duration
        cf.no_skipped_steps = options.no_skipped_steps
        cf.with_section_names = options.with_section_names
        cf.with_traceback = options.with_traceback
        cf.show_metrics = options.show_metrics

        # parse feature files
        fp = radish.FeatureParser()
        fp.parse()

        # load terrain and steps
        loader = radish.Loader(fp.get_features())
        loader.load()

        if cf.show_metrics:  # just get metrics of feature files
            ur = radish.UtilRegistry()
            if ur.has_util("show_metrics"):
                metrics = radish.Metrics(fp.get_features())
                try:
                    return ur.call_util("show_metrics", fp.get_features(), metrics.calculate())
                except KeyboardInterrupt:
                    pass
            else:
                raise radish.NoMetricUtilFoundError()
        else:  # normal run
            # run the features
            runner = radish.Runner(fp.get_features())
            endResult = runner.run()

            # report writer
            if cf.xunit_file:
                xunit = radish.XunitWriter(endResult)
                xunit.generate()

            exitCode = 0 if endResult.have_all_passed() else 1
    except radish.RadishError, e:
        sys.stderr.write("%s %s\n" % (radish.colorful.bold_red("radish error:"), e))
        exitCode = 2

    sys.exit(exitCode)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import sys

from radish.config import Config
from radish.hookregistry import HookRegistry
from radish.endresult import EndResult


class Runner(object):
    def __init__(self, features):
        self._features = features

    def run(self):
        hr = HookRegistry()
        hr.call_hook("before", "all")

        abort = False
        interrupted = False
        for f in self._features:
            hr.call_hook("before", "feature", f)

            for s in f.get_scenarios():
                hr.call_hook("before", "scenario", s)
                skip_remaining_steps = False

                for step in s.get_steps():
                    hr.call_hook("before", "step", step)

                    if not skip_remaining_steps and not interrupted:
                        try:
                            passed = step.run()
                            if not passed and not Config().dry_run:
                                skip_remaining_steps = True
                                if Config().abort_fail:
                                    abort = True
                        except KeyboardInterrupt:
                            interrupted = True
                            sys.stdout.write("\r")

                    hr.call_hook("after", "step", step)
                hr.call_hook("after", "scenario", s)
                if abort:  # if -a is set
                    break
            hr.call_hook("after", "feature", f)
            if abort:  # if -a is set
                break
        endResult = EndResult(self._features)
        hr.call_hook("after", "all", endResult)
        return endResult
